<?php

namespace Drupal\ics\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\image\ImageStyleInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'Image Comparison Slider' formatter.
 *
 * @FieldFormatter(
 *   id = "image_comparison_slider",
 *   label = @Translation("Image Comparison Slider"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class ImageComparisonSliderFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * EntityTypeManager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  private EntityTypeManager $entityTypeManager;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected FileUrlGeneratorInterface $fileUrlGenerator;

  /**
   * JuxtaposeBeforeAfterImageFormatter constructor.
   *
   * @param string $plugin_id
   *   Plugin id.
   * @param string $plugin_definition
   *   Plugin definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   Field definition.
   * @param array $settings
   *   Settings.
   * @param string $label
   *   Label.
   * @param string $view_mode
   *   View mode.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   *   EntityTypeManager.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, EntityTypeManager $entityTypeManager, FileUrlGeneratorInterface $file_url_generator) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->entityTypeManager = $entityTypeManager;
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * Creates an instance of the plugin.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The container to pull out services used in the plugin.
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   *
   * @return static
   *   Returns an instance of this plugin.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('file_url_generator')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'image_style' => '',
      'starting_position' => '',
      'additional_options' => '',
      'handle' => '',
      'styles' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();

    $image_style_options = [];
    $image_style_options['default'] = $this->t('Original size');

    try {
      $styles = $this->entityTypeManager->getStorage('image_style')
        ->loadMultiple();
      foreach ($styles as $key => $style) {
        $image_style_options[$key] = $style->label();
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('ics')->warning($e->getMessage());
    }

    $elements['image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Image style'),
      '#default_value' => $settings['image_style'],
      '#empty_value' => 'default',
      '#options' => $image_style_options,
    ];

    $elements['starting_position'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Starting position'),
      '#description' => $this->t('The starting position is expressed as a percentage, ex: 70.  Defaults to 50 if empty.'),
      '#default_value' => $settings['starting_position'],
      '#placeholder' => 'Ex: 30',
    ];

    $elements['additional_options'] = [
      '#type' => 'textarea',
      '#rows' => 5,
      '#title' => $this->t('Additional options'),
      '#description' => $this->t('Allows to define additional options for the ICS image formatter, ex: direction=vertical to slide between images vertically.'),
      '#default_value' => $settings['additional_options'],
      '#placeholder' => 'Ex: direction=vertical or hover=true',
    ];

    $elements['handle'] = [
      '#type' => 'textarea',
      '#rows' => 5,
      '#title' => $this->t('Custom SVG handle'),
      '#description' => $this->t('Allows to define a custom SVG handle, check examples documentation here:  https://img-comparison-slider.sneas.io/examples.html#custom-handle'),
      '#default_value' => $settings['handle'],
      '#placeholder' => 'Ex: <svg slot="handle" class="custom-handle" xmlns="http://www.w3.org/2000/svg" ...',
    ];

    $elements['styles'] = [
      '#type' => 'textarea',
      '#rows' => 15,
      '#title' => $this->t('Custom CSS'),
      '#description' => $this->t('Allows to make some quick CSS tweaks but should better be done in your site theme.  Examples available here: https://img-comparison-slider.sneas.io/examples.html'),
      '#default_value' => $settings['styles'],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $settings = $this->getSettings();
    $summary = [];

    if ($settings['image_style']) {
      $summary[] = $this->t('Image style: @image_style', ['@image_style' => $settings['image_style']]);
    }

    if ($settings['starting_position']) {
      $summary[] = $this->t('Starting position: @starting_position', ['@starting_position' => $settings['starting_position']]);
    }

    if (!empty($settings['handle'])) {
      $summary[] = $this->t('Custom handle: yes');
    }

    if (!empty($settings['styles'])) {
      $summary[] = $this->t('Custom CSS: yes');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $settings = $this->getSettings();

    $style = $this->getImageStyle($settings);

    $options = $this->getOptions($settings);

    $images = [];
    foreach ($items as $item) {
      $image = $this->getImageFromItem($item, $style);
      if (!is_null($image)) {
        $images[] = $image;
      }
      if (count($images) == 2) {
        break;
      }
    }

    return $this->buildRenderArray($images, $options);
  }

  /**
   * Fonction qui analyse une chaîne de caractères multiligne où chaque ligne
   * contient un couple clé-valeur séparé par un signe égal, et qui renvoie un
   * tableau associatif de ces clés et valeurs.
   *
   * @param $string
   *   La chaîne à analyser.
   *
   * @return array
   *   Le tableau associatif contenant les clés et les valeurs.
   */
  static function parseKeyValueString($string): array {
    $result = [];  // Tableau pour stocker les résultats
    $lines = explode("\n", $string);  // Sépare la chaîne en lignes

    foreach ($lines as $line) {
      if (trim($line) != '') {  // Vérifie si la ligne n'est pas vide
        [$key, $value] = explode('=', $line, 2);  // Sépare la ligne en deux au niveau du signe égal
        $key = trim($key);   // Nettoie les espaces blancs autour de la clé
        $value = trim($value);  // Nettoie les espaces blancs autour de la valeur
        if ($key != '') {
          $result[$key] = $value;  // Ajoute le couple clé-valeur au tableau résultat
        }
      }
    }

    return $result;  // Retourne le tableau associatif
  }

  /**
   * Build options array for rendering template.
   *
   * @param array $settings
   *   Formatter settings.
   *
   * @return array
   *   Options for template.
   */
  protected function getOptions(array $settings): array {
    $options = [];

    $options['starting_position'] = $settings['starting_position'] ?? '50%';
    $options['handle'] = !empty($settings['handle']) ? Xss::filter($settings['handle'], ['svg', 'path']) : '';
    $options['styles'] = !empty($settings['styles']) ? Xss::filter($settings['styles']) : '';

    $additional_options = self::parseKeyValueString($settings['additional_options']);

    return array_merge($options, $additional_options);
  }

  /**
   * Generate image data for the Twig template from image item.
   *
   * @param $image_item
   *   The image item.
   * @param \Drupal\image\ImageStyleInterface|null $style
   *   The image style to use for display.
   *
   * @return array|null
   *   The image data for the Twig template or NULL if file entity missing.
   */
  protected function getImageFromItem($image_item, ?ImageStyleInterface $style): array|null {
    $image_entity = $image_item->entity;
    if ($image_entity !== NULL) {
      $url = is_null($style)
        ? $this->fileUrlGenerator->generateAbsoluteString($image_entity->getFileUri())
        : $style->buildUrl($image_entity->getFileUri());

      $alt = $image_item->get('alt')->getValue();
      $title = $image_item->get('title')->getValue();

      $image = [];
      $image['url'] = $url;
      $image['alt'] = $alt;
      $image['caption'] = $title;

      return $image;
    }
    else {
      return NULL;
    }
  }

  /**
   * Load ImageStyle entity to use for display.
   *
   * @param array $settings
   *   Formatter settings.
   *
   * @return \Drupal\image\ImageStyleInterface|null
   */
  protected function getImageStyle(array $settings): ?ImageStyleInterface {
    try {
      return $this->entityTypeManager->getStorage('image_style')
        ->load($settings['image_style']);
    }
    catch (\Exception $e) {
      \Drupal::logger('ics')->warning($e->getMessage());
      return NULL;
    }
  }

  /**
   * Builds the CIS template render array.
   *
   * @param array $images
   *   Images array to add to the ICS slider.
   * @param array $options
   *   Configuration optiosn for the ICS slider.
   * @return array
   *   The Image Comparison Slider render array.
   */
  protected function buildRenderArray(array $images, array $options): array {
    return [
      '#theme' => 'image_comparison_slider',
      '#images' => $images,
      '#options' => $options,
      '#attached' => [
        'library' => [
          'ics/ics',
        ],
      ],
    ];
  }

}
